#include "mainwindow.h"
#include "ui_mainwindow.h"
#include "math.h"
#include "numeric"
#include "QDebug"
#include <QFile>
#include <QFileDialog>
#include <QTextStream>
#include <random>


MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);
}

MainWindow::~MainWindow()
{
    delete ui;
}

double F(double L, double KT, double SI, double KR, double p)
{
    double Val = (L*KT*SI*p*qPow(1+KT*SI*p,3)+KR*SI*p*qPow(1+KR*SI*p,3))/(L*qPow(1+KT*SI*p,4)+qPow(1+KR*SI*p,4));
    return (Val);
}

void MainWindow::on_btnCalculate_clicked()
{



    //Simulation parameters
    double dt=ui->eTimeStep->text().toDouble()/1000; //Time Step when integrating in s
    int df=round(1/dt); //Sanpling frequency in Hz
    int Len = ui->eDataLen->text().toInt()*df; //Overal lengths of simulated signals in time steps
    int Tran = ui->eTran->text().toInt()*df; //Duration of the transient process
    int dfW = ui->edfW->text().toInt(); //Частота дискретизации данных в файле
    dfW = round(df/dfW);
    qDebug() << "Проредить в " << dfW << " раз";

    //Prepare the files for data output
    //File to save the data in

    QString DirName = QFileDialog::getExistingDirectory();
    qDebug() << DirName;

    QFile f_ECG(DirName+"/ECG.txt"); //File for ECG
    QFile f_VsA(DirName+"/Alpha-sympathetic activity.txt"); //File for sympahetic activity
    QFile f_VsB(DirName+"/Beta-sympathetic activity.txt");
    QFile f_Vp(DirName+"/Parasympathetic activity.txt");
    QFile f_Vb_noise(DirName+"/Vb noise.txt"); // File for noise in barorceptors activity
    QFile f_Vp_noise(DirName+"/Vp noise.txt"); // File for noise in parasympathetic activity
    QFile f_VA(DirName+"/Lung volume.txt"); //File for lung ventilation
    QFile f_P(DirName+"/Arterial pressure.txt"); // File for arterial pressure


    f_ECG.open(QIODevice::ReadWrite);
    f_VsA.open(QIODevice::ReadWrite);
    f_VsB.open(QIODevice::ReadWrite);
    f_Vp.open(QIODevice::ReadWrite);
    f_Vb_noise.open(QIODevice::ReadWrite);
    f_Vp_noise.open(QIODevice::ReadWrite);
    f_VA.open(QIODevice::ReadWrite);
    f_P.open(QIODevice::ReadWrite);

    QTextStream ECG_stream(&f_ECG);
    QTextStream VsA_stream(&f_VsA);
    QTextStream VsB_stream(&f_VsB);
    QTextStream Vp_stream(&f_Vp);
    QTextStream Vp_noise_stream(&f_Vp_noise);
    QTextStream Vb_noise_stream(&f_Vb_noise);
    QTextStream VA_stream(&f_VA);
    QTextStream P_stream(&f_P);

    // Rhythm generation (pre-BötC) compartment
    QVector <double> t(50*df); t[0]=0;//Time, s
    QVector <double> A(50*df); A[0]=1;//Activity of the pre-BötC population of neurons

    QVector <double> hp(50*df); hp[0]=1;//Inactivation gating of persistent sodium

    double amp = 1; //Parameter affecting A
    double Gnap = ui->eGnap->text().toDouble(); //Activating rate constant of A, s^-1
    double GL = ui->eGL->text().toDouble(); //Inactivating rate constant of A, s^-1
    double Gn = ui->eGn->text().toDouble();;  //Nominal value of the control parameter Gt, s^-1
    double Gt = Gn;  //Value of the control parameter Gt, s^-1
    double b = ui->eb->text().toDouble(); //Scaling parameter in A
    double EL = ui->eEL->text().toDouble(); //Parameter affecting the external drive of A

    double FImp = ui->eFImp->text().toDouble(); //Parameter affecting amp
    double SImp =  ui->eSImp->text().toDouble();  //Parameter affecting amp

    double Ahp =1; //Parameter affecting hp
    double Bhp = 1; //Parameter affecting hp
    double FIhp = ui->eFIhp->text().toDouble(); //Parameter affecting hp
    double SIhp =  ui->eSIhp->text().toDouble();  //Parameter affecting hp
    double Thp =  ui->eThp->text().toDouble();  //Parameter affecting hp

    //VRG (integrator) compartment parameters
    QVector <double> Rp(50*df); Rp[0]=1;//Ramp signal from VRG
    double Tr1 = 0.35; //threshold for A above which integration starts
    double Tr2 = 0.35; //Rp is set to "0", when A is < Tr2
    double Tr3 = 0.5; //Threshold (Tr3) that could be set at a lower saturation point. This could represent an external
                      //signal that stops the integration but not reset it.
    double Tr4 = 0.0001; // threshold to ensure that if K is small enough integration will not occur
    double Eps = 0.00001;//Small value
    double Il = 0.99999; //Il is a constant (Il < 1 for a leaky integrator)
    double K = ui->eK->text().toDouble(); //K modulates the amplitude of breathing (main factor) - nominal value for Kt
    double Kt = K; //Kt modulates the amplitude of breathing (main factor)

    //Muscle compartment parameters
    QVector <double> xm(50*df); xm[0]=1;//Displacment of respiratory muscle
    QVector <double> PL(50*df); PL[0]=1;//The pleural pressure

    double k1 = ui->ek1->text().toDouble(); // Recoil rate constant of muscle
    double k2 = ui->ek2->text().toDouble(); // Conversion constant

    double kp = ui->ekp->text().toDouble(); // Conversion constant
    double Pm = ui->ePm->text().toDouble(); //The mouth pressure
    double PL0 = ui->ePL0->text().toDouble(); //The difference that normally exists between
                                              //atmospheric and pleural pressures

    //Lung compartment parameters
    double E = ui->eE->text().toDouble(); // The lung elastance
    double QA = 1; // net flux of gas into the alveoli
    double q = 1; // Air flow
    double R = ui->eR->text().toDouble(); // Airways resistance to flow
    double Dc = ui->eDc->text().toDouble(); // Diffusion capacity of CO2
    double Do = ui->eDo->text().toDouble(); // Diffusion capacity of O2
    double pac = 1; //alveolar partial pressures of carbon dioxide
    double pao = 1; //alveolar partial pressures of oxigen
    double pw = ui->epw->text().toDouble(); // The vapor pressure of water at 37°C
    QVector <double> PA(50*df); PA[0]=1;//The alveoli total pressure
    QVector <double> Vi(50*df); Vi[0]=1;// Inspired volume


    //Parameters in pc
    double SIc = ui->eSIc->text().toDouble(); // Solubility of CO2 in blood plasma
    double Vc = ui->eVc->text().toDouble(); // Volume of the lung capillaries
    double DEL = qPow(10,ui->eDel->text().toDouble()); // Acceleration rate of the chemical reaction in which bicarbonate (HCO3-) binds with H+ and H2O and CO2 are released inside of a red blood cell.
    double l2 = ui->el2->text().toDouble(); // Hydration reaction rates
    double r2 = ui->er2->text().toDouble(); // Dehydration reaction rates
    double H = qPow(10,ui->eH->text().toDouble()); // Concentration of H+
    double pc0 = ui->epc0->text().toDouble(); // Concentration of H+
    double pce = 1; //Blood partial pressures at the end of the capillaries
    QVector <double> pce_a(Len); pce_a[0] = 1;
    QVector <double> pc(Len); pc[0]=pc0;//The blood partial pressures at the end of the capillaries

    //Parameters in po
    double Th = ui->eTh->text().toDouble(); // Concentration of hemoglobin
    double SI = ui->eSI->text().toDouble(); // solubility of  O2 in blood plasma
    double dF = 1; //Derivative of Hemoglobin saturation fuction F
    double KT = ui->eKT->text().toDouble(); // Parameter in F
    double L = ui->eL->text().toDouble(); // Parameter in F
    double KR = ui->eKR->text().toDouble(); // Parameter in F
    double po0 = ui->epo0->text().toDouble(); // Concentration of H+
    double poe = 0; //Blood partial pressures at the end of the capillaries
    double Cu = ui->eCu->text().toDouble(); // Molar gas volume at 37C
    QVector <double> poe_a(50*df); poe_a[0] = 1;
    QVector <double> po(50*df); po[0]=po0;//The blood partial pressures at the end of the capillaries

    //Parameters in z
    QVector <double> z(50*df); z[0]= (46*SIc*r2)/(l2*H);// Concentrations of concentration of bicarbonate in the blood

    //Parameters in fc and fo
    QVector <double> VA(50*df); VA[0] = 0; // Lung volume
    double foi = 0; // Concentration of O2 in the inspired air
    double fci = 0; // Concentration of CO2 in the inspired air
    //double fod = 0; // Concentration of O2 in the dead space of the lungs
    //double fcd = 0; // Concentration of CO2 in the dead space of the lungs
    double fom = ui->efom->text().toDouble(); // Concentration of O2 in the mouth
    double fcm = ui->efcm->text().toDouble(); // Concentration of CO2 in the mouth

    QVector <double> fo(50*df); fo[0]=0;// Concentrations of oxygen
    QVector <double> fc(50*df); fc[0]=0;// Concentrations of carbon dioxide

    //Neural PI control
    double I_K = 0; //Integral of Erc and Ero Errors for the K control

    double A1 = ui->eA1->text().toDouble(); // Parameters in Gn PI control
    double B1 = ui->eB1->text().toDouble(); // Parameters in Gn PI control

    double A2 = ui->eA2->text().toDouble(); // Parameters in Gn PI control
    double B2 = ui->eB2->text().toDouble(); // Parameters in Gn PI control
    double C2 = ui->eC2->text().toDouble(); // Parameters in Gn PI control
    double D2 = ui->eD2->text().toDouble(); // Parameters in Gn PI control

    int T1 = round(ui->eT1->text().toDouble()*df); // Delay between changes in blood CO2 and central chemo-receptors
    QVector <double> Erc(50*df); Erc[0]=0;// Error in optimal value of CO2 pressure in blood
    QVector <double> Ero(50*df); Ero[0]=0;// Error in optimal value of O2 pressure in blood

    //Neural control
    QVector <double> PS(50*df); PS[0]=0; // Smuthed partial pressure of CO2 in blood
    QVector <double> PSR(50*df); PSR[0]=0; // Activity of the Pulmonary Streach Receptors
    QVector <double> CCR(50*df); CCR[0]=0; // Activity of the Central Chemo Receptors
    QVector <double> Crtd_CR(50*df); Crtd_CR[0] = 0; //Response of Carotid (O2) Chemo Receptors


    //Cardiovascular system
    //Heart
    double ti = 0; //Time since last heart contrastion
    double T_hr = ui->eT_hr->text().toDouble(); // Period (in seconds) of the denervated heart (is equivalent to 100 b/min)
    QVector <double> Fi(50*df); Fi[0] = 0; // Phase of a heart;

    //Arterial pressure
    double Tsys = ui->eTsys->text().toDouble(); //Duration of the systolic phase of respiration
    double D = 80; //Previous diastolic pressure
    double kPR = ui->ekPR->text().toDouble(); //Coefficient between VA and P
    QVector <double> P(50*df); P[0] = 1; // Arterial pressure in mmHg;

    //Arterial baroreceptors
    double E1 = ui->eE1->text().toDouble();
    double Vb_noise_cur = 0; //Current Vb noise, refreshed after each heart beat
    QVector <double>  Vb_noise(50*df); Vb_noise[0] = 0;//Brownian noise in Vb
    double Pd = ui->ePd->text().toDouble();
    double Ab = ui->eAb->text().toDouble();
    double Bb = ui->eBb->text().toDouble();
    double Cb = ui->eCb->text().toDouble();
    double Db = ui->eDb->text().toDouble();
    QVector <double> Vb(50*df); Vb[0] = 1; // Arterial pressure in mmHg;

    //Heart contractility
    double Si = 40; // Heart contractility;

    //Alpha-sympathetic activity
    double Vs0A = ui->eVs0A->text().toDouble();
    double kbsA = ui->ekbsA->text().toDouble();
    double kcsA = ui->ekcsA->text().toDouble();
    double krsA = ui->ekrsA->text().toDouble();
    QVector <double> VsA(50*df); VsA[0] = Vs0A; // Alpha-sympathetic activity

    //Beta-sympathetic activity
    double Vs0B = ui->eVs0B->text().toDouble();
    double kbsB = ui->ekbsB->text().toDouble();
    double kcsB = ui->ekcsB->text().toDouble();
    double krsB = ui->ekrsB->text().toDouble();
    QVector <double> VsB(50*df); VsB[0] = Vs0B; // Beta-sympathetic activity

    //Parasympathetic activity
    double E2 = ui->eE2->text().toDouble();
    QVector <double>  Vp_noise(50*df); Vp_noise[0] = 0; //Brownian noise in Vp
    double Vp0 = ui->eVp0->text().toDouble();
    double kbp = ui->ekbp->text().toDouble();
    double kcp = ui->ekcp->text().toDouble();
    double krp = ui->ekrp->text().toDouble();
    QVector <double> Vp(50*df); Vp[0] = Vp0; // parasympathetic activity

    //Noradreniline concentration
    double tc = ui->etc->text().toDouble();
    double tv = ui->etv->text().toDouble();
    double kCcs = ui->ekCcs->text().toDouble();
    double kCvs = ui->ekCvs->text().toDouble();
    int Fic = round(ui->eFic->text().toDouble()*df);
    int Fiv = round(ui->eFiv->text().toDouble()*df);

    QVector <double> Cc(50*df); Cc[0] = 1; // Noradreniline concentration in cardiac muscle
    QVector <double> Cv(50*df); Cv[0] = 1; // Noradreniline concentration in arteial vessels

    //Heart contractility
    double Ti = 1; //Duration of previos heart cycle, s
    double S0 = ui->eS0->text().toDouble();
    double kcS = ui->ekcS->text().toDouble();
    double ktS = ui->ektS->text().toDouble();
    double aS = ui->eaS->text().toDouble();
    double Sid = S0;
    double nS = ui->enS->text().toDouble();

    //Sympathetic and parasympathetic factors
    double kCFi = ui->ekCFi->text().toDouble();
    double aCc = ui->eaCc->text().toDouble();
    double nC = ui->enC->text().toDouble();

    double kpFi = ui->ekpFi->text().toDouble();
    double aVp = ui->eaVp->text().toDouble();
    double np = ui->enp->text().toDouble();
    int Fip = round(ui->eFip->text().toDouble()*df);
    double FFi = 0; //Phase effectivnes curve

    QVector <double> fs(50*df); fs[0] = 1; // Noradreniline concentration in cardiac muscle
    QVector <double> fp(50*df); fp[0] = 1; // Noradreniline concentration in cardiac muscle

    //AP factors
    double tv0 = ui->etv0->text().toDouble();
    double atv = ui->eatv->text().toDouble();
    double aCv = ui->eaCv->text().toDouble();
    double nv = ui->env->text().toDouble();
    QVector <double> Tv(50*df); Tv[0] = 1; // //Peripheral resistance

    //Output values
    double V_a = 0; //Average minute volume (l/min)
    double N_i = 0; //Number of inspirations
    double Avrg_i = 0; //Average depths of inspiration
    double average_pce = 0; // Mean blood concentration of CO2
    double average_poe = 0; // Mean blood concentration of O2
    double aTi = 0; //Average HR, b/min
    double N_hr = 0; //Number of contractions
    double aSP = 0; //Average SP, mmHg
    double aDP = 0; //Average DP, mmHg

    //Generating Gaussinan noise to be used to calculate the Brownian noise in Vb and Vp
    std::default_random_engine generator_Vb, generator_Vp; //Generators for gausian noise
    generator_Vb.seed(std::chrono::system_clock::now().time_since_epoch().count());
    generator_Vp.seed(std::chrono::system_clock::now().time_since_epoch().count());
    std::normal_distribution<double> dist_Vb(0, 1); //Seting the parpameters of the generator for E1 noise in Vb
    std::normal_distribution<double> dist_Vp(0, 1); //Seting the parpameters of the generator for E2 noise in Vp

    int si; //Counter 1-50 s
    int sl = 50*df-1; //Last index of a buffer
    int sp; //Previous index in a buffer
    for (int i=1; i<Len; ++i)
    {
        si = i%(50*df); //Counter in the buffer

        if (si == 0) //Index of the previous value in the buffer
        {
            sp = sl; //Case when the previous value (in time) is a last value in the buffer
        }
        else
        {
            sp = si-1;
        }
        //qDebug() << "si: " << si << " sl: " << sl << " sp: " << sp;
        t[si] = i*dt; //Time, s
        ti += 1; //Time since the last heart contraction
        //Modeling pre-BötC population of neurons
        double Alpha = Gnap*amp*hp[sp];
        double Beta = Gt+GL;
        double Gamma = b*Gt+EL*GL;

        A[si]=(Alpha*(1-A[sp])-Beta*A[sp]+Gamma)*dt+A[sp]; //Activity of the pre-BötC population of neurons


        amp=1/(1+exp((A[si]-FImp)/SImp)); //activation gating of persistent sodium current in steady state
        Ahp = 1/(2*Thp)*exp(-(A[si]-FIhp)/(2*SIhp)); //Parameter affecting hp
        Bhp = 1/(2*Thp)*exp((A[si]-FIhp)/(2*SIhp)); //Parameter affecting hp
        hp[si]=(Ahp*(1-hp[sp])-Bhp*hp[sp])*dt + hp[sp]; //Inactivation gating of persistent sodium

        //Modeling Ventral Respiratory Group (Integrator)
        if ((A[si]>Tr1)&&(Rp[sp]<Tr4))
        {
            Rp[si]=Kt; //Ramp signal from VRG
        }
        else if (A[si]>Tr1)
        {
            Rp[si]=Il*((A[sp]+A[si])*(dt/2)+Rp[sp]); //Ramp signal from VRG
        }
        else if (A[si]<=Tr2)
        {
            Rp[si]=0; //Ramp signal from VRG
        }
        else if ((Rp[sp]>Tr3)&&(((A[si]-A[sp])/dt)<Eps))
        {
            Rp[si]=Rp[sp]; //Ramp signal from VRG
        }


        //Modeling displacment of respirtory muscle
        xm[si]=(-k1*xm[sp]+k2*Rp[si])*dt+xm[sp];

        PL[si] = Pm - PL0 - kp*xm[si]; //The pleural pressure

        PA[si] = (((Pm*E)/PA[sp])*QA+(PL[si]-PL[sp])/dt)*dt + PA[sp]; // Alveoli total pressure
        VA[si] = (PA[si]-PL[si])/E; //Lung volume

//        q_prev = q;
        q = (Pm-PA[si])/R; //air flow  (q>0 - inspiration) (q<0 expiration)
        QA = q + Dc*(pc[sp]-pac) + Do*(po[sp]-pao); //net flux of gas into the alveoli


        pac = fc[sp]*(PA[si]-pw); //the alveolar partial pressures of carbon dioxide
        pao = fo[sp] *(PA[si]-pw); //the alveolar partial pressures of oxygen


        pc[si] = ((Dc/(SIc*Vc*Cu))*(pac-pc[sp])+((DEL*l2)/SIc)*(H*z[sp])-DEL*r2*pc[sp])*dt+pc[sp];
        po[si] = ((Do/(SI*Vc*Cu))*qPow((1+(4*(Th/SI)*dF)),-1)*(pao-po[sp]))*dt+po[sp];


        dF = (F(L,KT,SI,KR,po[si]) - F(L,KT,SI,KR,po[sp]))/(po[si]-po[sp]);

        z[si] = (DEL*(r2*SIc*pc[si]-l2*z[sp]*H))*dt + z[sp];


        fo[si] = ((1/VA[si])*(Do*(po[sp]-pao)+q*(foi-fo[sp]) - fo[sp]*(Dc*(pc[sp]-pac)+Do*(po[sp] - pao))))*dt + fo[sp]; //Concentrations of O2
        fc[si] = ((1/VA[si])*(Dc*(pc[sp]-pac)+q*(fci-fc[sp]) - fc[sp]*(Do*(po[sp]-pao)+Dc*(pc[sp] - pac))))*dt + fc[sp]; //Concentrations of CO2



        if (q>0)
        {
            foi = fom;
            fci = fcm;
        }
        else if (q <= 0)
        {
            foi = fo[si];
            fci = fc[si];
        }

        if (Pm >= PA[si])
        {
            Vi[si] = ((2*Pm-PA[si]-PA[sp])/(2*R))*dt + Vi[sp]; //Volume of the inspired air
        }
        else if (Pm < PA[si])
        {
            Vi[si] = 0;
            if (Pm >= PA[sp] && i>400*df && Vi[sp]>0.025)
            {
                V_a += Vi[sp];
                N_i += 1;
            }
        }




        if (i > T1)
        {

            if (T1>si)
                Erc[si] = 0.585*tanh((pce_a[si+sl-T1]-61)/20.7) + 0.442; //Error in CO2 concentration
            else
                Erc[si] = 0.585*tanh((pce_a[si-T1]-61)/20.7) + 0.442; //Error in CO2 concentration

            if (poe_a[si] <= 100)
            {
                Crtd_CR[si] = -0.5*tanh((poe_a[sp]-50)/20)+0.496; //Error in O2 concentration
            }
            else if (poe_a[si] > 100)
                Crtd_CR[si] = -0.0005*poe_a[sp]+0.053;

            I_K += C2*dt*((Erc[si]+Erc[sp])/2) - D2*dt*((Crtd_CR[si] + Crtd_CR[sp])/2); //Integral of Erc and Ero Errors for the K control

            Gt = Gn + A1*Erc[si] + B1*Crtd_CR[si]; //Controls the frequency and the amplitude (Higher Gt - higher frequency, lower A)
            Kt = K + A2*Erc[si] + B2*Crtd_CR[si] + I_K; //Controls only the amplitude (Higher Kt - higher A)
        }
        else if (i <= T1)
        {
            Gt = Gn;
            Kt = K;
        }

        //CVS compartments
        //Heart
        Fi[si] = (1/T_hr)*fs[sp]*fp[sp]*dt+Fi[sp];

        pce_a[si] = pce;
        poe_a[si] = poe;
        if (Fi[si] >= 1 && Fi[sp]<1) //Heart contracted/new heart cycle
        {
            Vb_noise_cur += E1*dist_Vb(generator_Vb); //Brownian noise in Vb, set after each heart contraction
            //qDebug() << Vb_noise_cur;
            Fi[si] = 0;
            D = P[sp];

            //Reseting the parameters of respiration
            pce = pc[sp];
            poe = po[sp];

            pc[si]=pc0;
            po[si]=po0;
            z[si] = (46*SIc*r2)/(l2*H);
            Ti = ti*dt;

            if (i > Tran)
            {
                aTi += Ti; //Calculating average HR
                N_hr += 1; //Nuber of heart contractions
                aDP += P[sp];
            }
            ti = 0;
        }
        if (i > Tran)
        {
            average_pce += pce_a[sp];
            average_poe += poe_a[sp];
        }
        //Arterial pressure
        if (ti*dt <= Tsys)
        {
            P[si] = D + Si*(ti*dt/Tsys)*exp(1-(ti*dt/Tsys))+kPR*VA[si];
        }
        else if (ti*dt > Tsys)
        {
            P[si] = (-P[sp]/Tv[sp])*dt+P[sp];

            if (((ti-1)*dt <= Tsys)&&(i > Tran))
            {
                aSP += P[sp];
            }
        }

        //Activity of the carotid baroreceptors
        Vb_noise[si] = Vb_noise_cur;
        if (P[si] >= 50)
        {
            Vb[si] = Ab*tanh((P[si]+Pd*(P[si]-P[sp])/dt - Bb)/Cb)+Db + Vb_noise[si];

        }
        else if (P[si] < 50)
        {
            Vb[si] = 0;
        }


        //Alpha-sympathetic activity (Affects vascular tone)

        VsA[si] = Vs0A - kbsA*Vb[si] + kcsA*Erc[si] - krsA*VA[si]; //VsA - Alpha-sympathetic activity
        if (VsA[si] < 0)
        {
            VsA[si] = 0;
        }

        VsB[si] = Vs0B - kbsB*Vb[si] + kcsB*Erc[si] + krsB*VA[si]; //VsB - Beta-sympathetic activity
        if (VsB[si] < 0)
        {
            VsB[si] = 0;
        }

        Vp[si] = Vp0 + kbp*Vb[si]+kcp*Erc[si] - krp*VA[si] + E2*Vp_noise[si]; //Vp - parasympathetic activity
        Vp_noise[si] = 0.97*Vp_noise[sp] + dist_Vp(generator_Vp);
        if (Vp[si] < 0)
        {
            Vp[si] = 0;
        }

        if (i>=Fic)
        {
            if (Fic>si)
                 Cc[si] = (-Cc[sp]/tc+kCcs*VsB[si+sl-Fic])*dt+Cc[sp]; //Noradremaline concentration in cardiac tissue
            else
                Cc[si] = (-Cc[sp]/tc+kCcs*VsB[si-Fic])*dt+Cc[sp]; //Noradremaline concentration in cardiac tissue
        }
        else
        {
            Cc[si]=aCc;
        }

        if (i>=Fiv)
        {
            if (Fiv>si)
                Cv[si] = (-Cv[sp]/tv+kCvs*VsA[si+sl-Fiv])*dt+Cv[sp]; //Noradremaline concentration in arterial vessels
            else
                Cv[si] = (-Cv[sp]/tv+kCvs*VsA[si-Fiv])*dt+Cv[sp]; //Noradremaline concentration in arterial vessels

        }
        else
        {
            Cv[si]=aCv;
        }



        Sid = S0 + kcS*Cc[si] + ktS*Ti; // Heart Contractility
        Si = Sid + (aS-Sid)*((pow(Sid,nS)/(pow(Sid,nS)+pow(aS,nS))));

        fs[si]= 1 +kCFi*(Cc[si]+(aCc-Cc[si])*(pow(Cc[si],nC)/(pow(aCc,nC)+pow(Cc[si],nC))));

        FFi = pow(Fi[si],1.3)*(Fi[si]-0.45)*(pow(1-Fi[si],3)/(pow(1-0.8,3)+pow(1-Fi[si],3)));

        if (i > Fip)
        {
            if (Fip>si)
                fp[si]= 1 - kpFi*(Vp[si+sl-Fip]+(aVp-Vp[si+sl-Fip])*(pow(Vp[si+sl-Fip],np)/(pow(aVp,np)+pow(Vp[si+sl-Fip],np))))*FFi;
            else
                fp[si]= 1 - kpFi*(Vp[si-Fip]+(aVp-Vp[si-Fip])*(pow(Vp[si-Fip],np)/(pow(aVp,np)+pow(Vp[si-Fip],np))))*FFi;
        }


        //AP control
        Tv[si] = tv0-atv*(Cv[si]+(aCv-Cv[si])*(pow(Cv[si],nv)/(pow(aCv,nv)+pow(Cv[si],nv))));



        if ((ui->cbWrite->isChecked() == 1)&&(i%dfW == 0)&&(i>Tran))
        {
          //qDebug() << "Writing i " << i;

          ECG_stream << Fi[si] << Qt::endl;
          VsA_stream << VsA[si] << Qt::endl;
          VsB_stream << VsB[si] << Qt::endl;
          Vp_stream << Vp[si] << Qt::endl;
          Vb_noise_stream << Vb_noise[si] << Qt::endl;
          Vp_noise_stream << E2*Vp_noise[si] << Qt::endl;
          VA_stream << VA[si] << Qt::endl;
          P_stream << P[si] << Qt::endl;
        }

    }
    //Arterial pressure
    //Autonomic control of heart rate


    Avrg_i = V_a/N_i; //Average depths of respiration
    V_a = 60*V_a/((Len-Tran)*dt); //Calculating minute volume
    N_i = N_i/((Len-Tran)*dt); // Rate of respiration
    average_pce /= (Len-Tran); //Average Blod CO2 mmHg
    average_poe /= (Len-Tran);  //Average Blod O2 mmHg
    aTi = N_hr/aTi; //Average heart rate Hz
    aSP /= N_hr; //Average diastolic pressure
    aDP /= N_hr; //Average diastolic pressure

    ui->eOutput->clear();
    ui->eOutput->append("Minute volume (l/min): " + QString::number(V_a));
    ui->eOutput->append("Blod CO2 mmHg: " + QString::number(average_pce));
    ui->eOutput->append("Blod O2 mmHg: " +  QString::number(average_poe));
    ui->eOutput->append("Respiratory Rate, Hz: " + QString::number(N_i));
    ui->eOutput->append("Depths of Respiration, l: " + QString::number(Avrg_i));
    ui->eOutput->append("Heart rate, Hz: " + QString::number(aTi));
    ui->eOutput->append("Systolic pressure, mmHg: " + QString::number(aSP));
    ui->eOutput->append("Diastolic pressure, mmHg: " + QString::number(aDP));


    ui->customPlotA->setInteraction(QCP::iRangeDrag, true);
    ui->customPlotA->setInteraction(QCP::iRangeZoom, true);
    ui->customPlotA->addGraph();
    ui->customPlotA->graph(0)->setData(t,A);
    ui->customPlotA->xAxis->setLabel("t, s");
    ui->customPlotA->yAxis->setLabel("A(t)");
    ui->customPlotA->graph(0)->rescaleAxes();
    ui->customPlotA->replot();

    ui->customPlotRp->setInteraction(QCP::iRangeDrag, true);
    ui->customPlotRp->setInteraction(QCP::iRangeZoom, true);
    ui->customPlotRp->addGraph();
    ui->customPlotRp->graph(0)->setData(t,VA);
    ui->customPlotRp->xAxis->setLabel("t, s");
    ui->customPlotRp->yAxis->setLabel("Lung Volume, l");
    ui->customPlotRp->graph(0)->rescaleAxes();
    ui->customPlotRp->replot();


    ui->customPlotVA->setInteraction(QCP::iRangeDrag, true);
    ui->customPlotVA->setInteraction(QCP::iRangeZoom, true);
    ui->customPlotVA->addGraph();
    ui->customPlotVA->graph(0)->setData(t,Fi);
    ui->customPlotVA->xAxis->setLabel("t, s");
    ui->customPlotVA->yAxis->setLabel("Heart rate, l");
    ui->customPlotVA->graph(0)->rescaleAxes();
    ui->customPlotVA->replot();

    ui->customPlotPA->setInteraction(QCP::iRangeDrag, true);
    ui->customPlotPA->setInteraction(QCP::iRangeZoom, true);
    ui->customPlotPA->addGraph();
    ui->customPlotPA->graph(0)->setData(t,P);
    ui->customPlotPA->xAxis->setLabel("t, s");
    ui->customPlotPA->yAxis->setLabel("Arterial Pressure, mmHg");
    ui->customPlotPA->graph(0)->rescaleAxes();
    ui->customPlotPA->replot();


    ui->customPlotPo->setInteraction(QCP::iRangeDrag, true);
    ui->customPlotPo->setInteraction(QCP::iRangeZoom, true);
    ui->customPlotPo->addGraph();
    ui->customPlotPo->graph(0)->setData(t,VsA);
    ui->customPlotPo->xAxis->setLabel("t, s");
    ui->customPlotPo->yAxis->setLabel("Alpha-sympathetic activity");
    ui->customPlotPo->graph(0)->rescaleAxes();
    ui->customPlotPo->replot();

    ui->customPlotPc->setInteraction(QCP::iRangeDrag, true);
    ui->customPlotPc->setInteraction(QCP::iRangeZoom, true);
    ui->customPlotPc->addGraph();
    ui->customPlotPc->graph(0)->setData(t,VsB);
    ui->customPlotPc->xAxis->setLabel("t, s");
    ui->customPlotPc->yAxis->setLabel("Beta-sympathetic activity");
    ui->customPlotPc->graph(0)->rescaleAxes();
    ui->customPlotPc->replot();

    f_ECG.close();
    f_VsA.close();
    f_VsB.close();
    f_Vp.close();
    f_Vp_noise.close();
    f_Vb_noise.close();
    f_VA.close();
    f_P.close();

}


void MainWindow::on_bWrite_clicked()
{

}
